@@ -117,6 +117,8 @@ public class ShuffleClientImpl extends ShuffleClient {
   private final Set<String> pushExcludedWorkers = ConcurrentHashMap.newKeySet();
   private final ConcurrentHashMap<String, Long> fetchExcludedWorkers =
       JavaUtils.newConcurrentHashMap();
+  private boolean pushReplicateEnabled;
+  private boolean fetchExcludeWorkerOnFailureEnabled;
 
   private final ExecutorService pushDataRetryPool;
 
@@ -136,7 +138,7 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   private final ReviveManager reviveManager;
 
-  protected static class ReduceFileGroups {
+  public static class ReduceFileGroups {
     public Map<Integer, Set<PartitionLocation>> partitionGroups;
     public int[] mapAttempts;
     public Set<Integer> partitionIds;
@@ -179,6 +181,8 @@ public class ShuffleClientImpl extends ShuffleClient {
     pushBufferMaxSize = conf.clientPushBufferMaxSize();
     pushExcludeWorkerOnFailureEnabled = conf.clientPushExcludeWorkerOnFailureEnabled();
     shuffleCompressionEnabled = !conf.shuffleCompressionCodec().equals(CompressionCodec.NONE);
+    pushReplicateEnabled = conf.clientPushReplicateEnabled();
+    fetchExcludeWorkerOnFailureEnabled = conf.clientFetchExcludeWorkerOnFailureEnabled();
     if (conf.clientPushReplicateEnabled()) {
       pushDataTimeout = conf.pushDataTimeoutMs() * 2;
     } else {
@@ -579,7 +583,8 @@ public class ShuffleClientImpl extends ShuffleClient {
   }
 
   @Override
-  public int getShuffleId(int appShuffleId, String appShuffleIdentifier, boolean isWriter) {
+  public int getShuffleId(
+      int appShuffleId, String appShuffleIdentifier, boolean isWriter, boolean isBarrierStage) {
     return shuffleIdCache.computeIfAbsent(
         appShuffleIdentifier,
         (id) -> {
@@ -588,6 +593,7 @@ public class ShuffleClientImpl extends ShuffleClient {
                   .setAppShuffleId(appShuffleId)
                   .setAppShuffleIdentifier(appShuffleIdentifier)
                   .setIsShuffleWriter(isWriter)
+                  .setIsBarrierStage(isBarrierStage)
                   .build();
           PbGetShuffleIdResponse pbGetShuffleIdResponse =
               lifecycleManagerRef.askSync(
@@ -613,6 +619,20 @@ public class ShuffleClientImpl extends ShuffleClient {
     return pbReportShuffleFetchFailureResponse.getSuccess();
   }
 
+  public boolean reportBarrierTaskFailure(int appShuffleId, String appShuffleIdentifier) {
+    PbReportBarrierStageAttemptFailure pbReportBarrierStageAttemptFailure =
+        PbReportBarrierStageAttemptFailure.newBuilder()
+            .setAppShuffleId(appShuffleId)
+            .setAppShuffleIdentifier(appShuffleIdentifier)
+            .build();
+    PbReportBarrierStageAttemptFailureResponse pbReportBarrierStageAttemptFailureResponse =
+        lifecycleManagerRef.askSync(
+            pbReportBarrierStageAttemptFailure,
+            conf.clientRpcRegisterShuffleAskTimeout(),
+            ClassTag$.MODULE$.apply(PbReportBarrierStageAttemptFailureResponse.class));
+    return pbReportBarrierStageAttemptFailureResponse.getSuccess();
+  }
+
   private ConcurrentHashMap<Integer, PartitionLocation> registerShuffleInternal(
       int shuffleId,
       int numMappers,
@@ -873,7 +893,8 @@ public class ShuffleClientImpl extends ShuffleClient {
       int length,
       int numMappers,
       int numPartitions,
-      boolean doPush)
+      boolean doPush,
+      boolean skipCompress)
       throws IOException {
     // mapKey
     final String mapKey = Utils.makeMapKey(shuffleId, mapId, attemptId);
@@ -938,7 +959,7 @@ public class ShuffleClientImpl extends ShuffleClient {
     // increment batchId
     final int nextBatchId = pushState.nextBatchId();
 
-    if (shuffleCompressionEnabled) {
+    if (shuffleCompressionEnabled && !skipCompress) {
       // compress data
       final Compressor compressor = compressorThreadLocal.get();
       compressor.compress(data, offset, length);
@@ -1244,7 +1265,8 @@ public class ShuffleClientImpl extends ShuffleClient {
         length,
         numMappers,
         numPartitions,
-        true);
+        true,
+        false);
   }
 
   @Override
@@ -1278,6 +1300,7 @@ public class ShuffleClientImpl extends ShuffleClient {
         length,
         numMappers,
         numPartitions,
+        false,
         false);
   }
 
@@ -1840,6 +1863,8 @@ public class ShuffleClientImpl extends ShuffleClient {
       cause = StatusCode.PUSH_DATA_PRIMARY_WORKER_EXCLUDED;
     } else if (message.startsWith(StatusCode.PUSH_DATA_REPLICA_WORKER_EXCLUDED.name())) {
       cause = StatusCode.PUSH_DATA_REPLICA_WORKER_EXCLUDED;
+    } else if (message.startsWith(StatusCode.PUSH_DATA_FAIL_PARTITION_NOT_FOUND.name())) {
+      cause = StatusCode.PUSH_DATA_FAIL_PARTITION_NOT_FOUND;
     } else if (ExceptionUtils.connectFail(message)) {
       // Throw when push to primary worker connection causeException.
       cause = StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_PRIMARY;
@@ -1853,4 +1878,12 @@ public class ShuffleClientImpl extends ShuffleClient {
   public TransportClientFactory getDataClientFactory() {
     return dataClientFactory;
   }
+
+  public void excludeFailedFetchLocation(String hostAndFetchPort, Exception e) {
+    if (pushReplicateEnabled
+        && fetchExcludeWorkerOnFailureEnabled
+        && Utils.isCriticalCauseForFetch(e)) {
+      fetchExcludedWorkers.put(hostAndFetchPort, System.currentTimeMillis());
+    }
+  }
 }
