@@ -27,8 +27,10 @@ import java.util.concurrent.atomic.LongAdder;
 
 import scala.Tuple2;
 
+import com.github.luben.zstd.ZstdException;
 import com.google.common.util.concurrent.Uninterruptibles;
 import io.netty.buffer.ByteBuf;
+import net.jpountz.lz4.LZ4Exception;
 import org.roaringbitmap.RoaringBitmap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -157,8 +159,6 @@ public abstract class CelebornInputStream extends InputStream {
     private final boolean enabledReadLocalShuffle;
     private final String localHostAddress;
 
-    private boolean pushReplicateEnabled;
-    private boolean fetchExcludeWorkerOnFailureEnabled;
     private boolean shuffleCompressionEnabled;
     private long fetchExcludedWorkerExpireTimeout;
     private ConcurrentHashMap<String, Long> fetchExcludedWorkers;
@@ -203,8 +203,6 @@ public abstract class CelebornInputStream extends InputStream {
       this.rangeReadFilter = conf.shuffleRangeReadFilterEnabled();
       this.enabledReadLocalShuffle = conf.enableReadLocalShuffleFile();
       this.localHostAddress = Utils.localHostName(conf);
-      this.pushReplicateEnabled = conf.clientPushReplicateEnabled();
-      this.fetchExcludeWorkerOnFailureEnabled = conf.clientFetchExcludeWorkerOnFailureEnabled();
       this.shuffleCompressionEnabled =
           !conf.shuffleCompressionCodec().equals(CompressionCodec.NONE);
       this.fetchExcludedWorkerExpireTimeout = conf.clientFetchExcludedWorkerExpireTimeout();
@@ -297,12 +295,6 @@ public abstract class CelebornInputStream extends InputStream {
       }
     }
 
-    private void excludeFailedLocation(PartitionLocation location, Exception e) {
-      if (pushReplicateEnabled && fetchExcludeWorkerOnFailureEnabled && isCriticalCause(e)) {
-        fetchExcludedWorkers.put(location.hostAndFetchPort(), System.currentTimeMillis());
-      }
-    }
-
     private boolean isExcluded(PartitionLocation location) {
       Long timestamp = fetchExcludedWorkers.get(location.hostAndFetchPort());
       if (timestamp == null) {
@@ -352,7 +344,7 @@ public abstract class CelebornInputStream extends InputStream {
           return createReader(location, pbStreamHandler, fetchChunkRetryCnt, fetchChunkMaxRetry);
         } catch (Exception e) {
           lastException = e;
-          excludeFailedLocation(location, e);
+          shuffleClient.excludeFailedFetchLocation(location.hostAndFetchPort(), e);
           fetchChunkRetryCnt++;
           if (location.hasPeer()) {
             // fetchChunkRetryCnt % 2 == 0 means both replicas have been tried,
@@ -390,7 +382,8 @@ public abstract class CelebornInputStream extends InputStream {
           }
           return currentReader.next();
         } catch (Exception e) {
-          excludeFailedLocation(currentReader.getLocation(), e);
+          shuffleClient.excludeFailedFetchLocation(
+              currentReader.getLocation().hostAndFetchPort(), e);
           fetchChunkRetryCnt++;
           currentReader.close();
           if (fetchChunkRetryCnt == fetchChunkMaxRetry) {
@@ -664,8 +657,20 @@ public abstract class CelebornInputStream extends InputStream {
         }
 
         return hasData;
-      } catch (IOException e) {
-        IOException ioe = e;
+      } catch (LZ4Exception | ZstdException | IOException e) {
+        logger.error(
+            "Failed to fill buffer from chunk. AppShuffleId {}, shuffleId {}, partitionId {}, location {}",
+            appShuffleId,
+            shuffleId,
+            partitionId,
+            Optional.ofNullable(currentReader).map(PartitionReader::getLocation).orElse(null),
+            e);
+        IOException ioe;
+        if (e instanceof IOException) {
+          ioe = (IOException) e;
+        } else {
+          ioe = new IOException(e);
+        }
         if (exceptionMaker != null) {
           if (shuffleClient.reportShuffleFetchFailure(appShuffleId, shuffleId)) {
             /*
@@ -680,6 +685,15 @@ public abstract class CelebornInputStream extends InputStream {
           }
         }
         throw ioe;
+      } catch (Exception e) {
+        logger.error(
+            "Failed to fill buffer from chunk. AppShuffleId {}, shuffleId {}, partitionId {}, location {}",
+            appShuffleId,
+            shuffleId,
+            partitionId,
+            Optional.ofNullable(currentReader).map(PartitionReader::getLocation).orElse(null),
+            e);
+        throw e;
       }
     }
 
