@@ -173,7 +173,8 @@ public class DummyShuffleClient extends ShuffleClient {
   }
 
   @Override
-  public int getShuffleId(int appShuffleId, String appShuffleIdentifier, boolean isWriter) {
+  public int getShuffleId(
+      int appShuffleId, String appShuffleIdentifier, boolean isWriter, boolean isBarrierStage) {
     return appShuffleId;
   }
 
@@ -182,11 +183,18 @@ public class DummyShuffleClient extends ShuffleClient {
     return true;
   }
 
+  public boolean reportBarrierTaskFailure(int appShuffleId, String appShuffleIdentifier) {
+    return true;
+  }
+
   @Override
   public TransportClientFactory getDataClientFactory() {
     return null;
   }
 
+  @Override
+  public void excludeFailedFetchLocation(String hostAndFetchPort, Exception e) {}
+
   public void initReducePartitionMap(int shuffleId, int numPartitions, int workerNum) {
     ConcurrentHashMap<Integer, PartitionLocation> map = JavaUtils.newConcurrentHashMap();
     String host = "host";
