@@ -366,7 +366,7 @@ public class TransportClient implements Closeable {
   @Override
   public String toString() {
     return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
-        .append("remoteAdress", channel.remoteAddress())
+        .append("remoteAddress", channel.remoteAddress())
         .append("clientId", clientId)
         .append("isActive", isActive())
         .toString();
@@ -393,7 +393,8 @@ public class TransportClient implements Closeable {
    * <p>Trying to set a different client ID after it's been set will result in an exception.
    */
   public void setClientId(String id) {
-    Preconditions.checkState(clientId == null, "Client ID has already been set.");
+    Preconditions.checkState(
+        clientId == null || clientId.equals(id), "Client ID has already been set.");
     this.clientId = id;
   }
 
