@@ -526,6 +526,14 @@ object ControlMessages extends Logging {
     case pb: PbReportShuffleFetchFailureResponse =>
       new TransportMessage(MessageType.REPORT_SHUFFLE_FETCH_FAILURE_RESPONSE, pb.toByteArray)
 
+    case pb: PbReportBarrierStageAttemptFailure =>
+      new TransportMessage(MessageType.REPORT_BARRIER_STAGE_ATTEMPT_FAILURE, pb.toByteArray)
+
+    case pb: PbReportBarrierStageAttemptFailureResponse =>
+      new TransportMessage(
+        MessageType.REPORT_BARRIER_STAGE_ATTEMPT_FAILURE_RESPONSE,
+        pb.toByteArray)
+
     case HeartbeatFromWorker(
           host,
           rpcPort,
@@ -1275,6 +1283,12 @@ object ControlMessages extends Logging {
 
       case APPLICATION_META_REQUEST_VALUE =>
         PbApplicationMetaRequest.parseFrom(message.getPayload)
+
+      case REPORT_BARRIER_STAGE_ATTEMPT_FAILURE_VALUE =>
+        PbReportBarrierStageAttemptFailure.parseFrom(message.getPayload)
+
+      case REPORT_BARRIER_STAGE_ATTEMPT_FAILURE_RESPONSE_VALUE =>
+        PbReportBarrierStageAttemptFailureResponse.parseFrom(message.getPayload)
     }
   }
 }
