@@ -305,12 +305,12 @@ object CelebornCommonSettings {
       }
     },
     licenses += ("Apache-2.0", url("https://www.apache.org/licenses/LICENSE-2.0")),
-      pomExtra :=
-        <url>https://celeborn.apache.org/</url>
-        <scm>
-          <url>git@github.com:apache/celeborn.git</url>
-          <connection>scm:git:git@github.com:apache/celeborn.git</connection>
-        </scm>
+    scmInfo := Some(
+      ScmInfo(
+        url("https://celeborn.apache.org/"),
+          "scm:git:https://github.com/apache/celeborn.git",
+          "scm:git:git@github.com:apache/celeborn.git"))
+
   )
 
   lazy val protoSettings = Seq(
@@ -318,7 +318,7 @@ object CelebornCommonSettings {
     PB.protocVersion := Dependencies.protocVersion,
     // set proto sources path
     Compile / PB.protoSources := Seq(sourceDirectory.value / "main" / "proto"),
-    Compile / PB.targets := Seq(PB.gens.java -> (Compile / sourceManaged).value)
+    Compile / PB.targets := Seq(PB.gens.java(Dependencies.protocVersion) -> (Compile / sourceManaged).value)
   )
 
   lazy val commonUnitTestDependencies = Seq(
