@@ -462,7 +462,8 @@ public class MemoryManager {
     readBufferCounter.addAndGet(delta);
   }
 
-  protected boolean readBufferAvailable(int requiredBytes) {
+  @VisibleForTesting
+  public boolean readBufferAvailable(int requiredBytes) {
     return readBufferCounter.get() + requiredBytes < readBufferThreshold;
   }
 
@@ -527,7 +528,7 @@ public class MemoryManager {
     return memoryFileStorageCounter.sum() < memoryFileStorageThreshold;
   }
 
-  public void increaseMemoryFileStorage(int bytes) {
+  public void incrementMemoryFileStorage(int bytes) {
     memoryFileStorageCounter.add(bytes);
   }
 
