@@ -298,15 +298,6 @@ public abstract class PartitionDataWriter implements DeviceObserver {
     }
 
     final int numBytes = data.readableBytes();
-    if (isMemoryShuffleFile.get()) {
-      MemoryManager.instance().increaseMemoryFileStorage(numBytes);
-    } else {
-      MemoryManager.instance().incrementDiskBuffer(numBytes);
-      if (userBufferInfo != null) {
-        userBufferInfo.updateInfo(
-            System.currentTimeMillis(), new BufferStatusHub.BufferStatusNode(numBytes));
-      }
-    }
 
     synchronized (flushLock) {
       if (closed) {
@@ -334,6 +325,17 @@ public abstract class PartitionDataWriter implements DeviceObserver {
         }
       }
 
+      // update the disk buffer or memory file storage after evict
+      if (isMemoryShuffleFile.get()) {
+        MemoryManager.instance().incrementMemoryFileStorage(numBytes);
+      } else {
+        MemoryManager.instance().incrementDiskBuffer(numBytes);
+        if (userBufferInfo != null) {
+          userBufferInfo.updateInfo(
+              System.currentTimeMillis(), new BufferStatusHub.BufferStatusNode(numBytes));
+        }
+      }
+
       data.retain();
       flushBuffer.addComponent(true, data);
       if (isMemoryShuffleFile.get()) {
@@ -523,7 +525,7 @@ public abstract class PartitionDataWriter implements DeviceObserver {
     }
   }
 
-  protected FileInfo getCurrentFileInfo() {
+  public FileInfo getCurrentFileInfo() {
     if (!isMemoryShuffleFile.get()) {
       return diskFileInfo;
     } else {
@@ -553,7 +555,7 @@ public abstract class PartitionDataWriter implements DeviceObserver {
       waitTime -= WAIT_INTERVAL_MS;
     }
     if (counter.get() > 0) {
-      IOException ioe = new IOException("Wait pending actions timeout.");
+      IOException ioe = new IOException("Wait pending actions timeout, Counter: " + counter.get());
       notifier.setException(ioe);
       throw ioe;
     }
