@@ -31,14 +31,14 @@ import org.roaringbitmap.RoaringBitmap
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{WorkerInfo, WorkerPartitionLocationInfo}
+import org.apache.celeborn.common.meta.{ReduceFileMeta, WorkerInfo, WorkerPartitionLocationInfo}
 import org.apache.celeborn.common.metrics.MetricsSystem
 import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType, StorageInfo}
 import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.rpc._
 import org.apache.celeborn.common.util.{JavaUtils, Utils}
-import org.apache.celeborn.service.deploy.worker.storage.{FileWriter, MapPartitionFileWriter, StorageManager}
+import org.apache.celeborn.service.deploy.worker.storage.{MapPartitionDataWriter, PartitionDataWriter, StorageManager}
 
 private[deploy] class Controller(
     override val rpcEnv: RpcEnv,
@@ -89,6 +89,7 @@ private[deploy] class Controller(
           userIdentifier,
           pushDataTimeout,
           partitionSplitEnabled) =>
+      checkAuth(context, applicationId)
       val shuffleKey = Utils.makeShuffleKey(applicationId, shuffleId)
       workerSource.sample(WorkerSource.RESERVE_SLOTS_TIME, shuffleKey) {
         logDebug(s"Received ReserveSlots request, $shuffleKey, " +
@@ -118,6 +119,7 @@ private[deploy] class Controller(
           mapAttempts,
           epoch,
           mockFailure) =>
+      checkAuth(context, applicationId)
       val shuffleKey = Utils.makeShuffleKey(applicationId, shuffleId)
       logDebug(s"Received CommitFiles request, $shuffleKey, primary files" +
         s" ${primaryIds.asScala.mkString(",")}; replica files ${replicaIds.asScala.mkString(",")}.")
@@ -135,6 +137,7 @@ private[deploy] class Controller(
         s"$commitFilesTimeMs ms.")
 
     case DestroyWorkerSlots(shuffleKey, primaryLocations, replicaLocations, mockFailure) =>
+      checkAuth(context, Utils.splitShuffleKey(shuffleKey)._1)
       handleDestroy(context, shuffleKey, primaryLocations, replicaLocations, mockFailure)
   }
 
@@ -173,7 +176,7 @@ private[deploy] class Controller(
           requestPrimaryLocs.get(ind).getUniqueId)
         if (location == null) {
           location = requestPrimaryLocs.get(ind)
-          val writer = storageManager.createWriter(
+          val writer = storageManager.createPartitionDataWriter(
             applicationId,
             shuffleId,
             location,
@@ -197,8 +200,8 @@ private[deploy] class Controller(
       logWarning(s"[handleReserveSlots] $msg, will destroy writers.")
       primaryLocs.asScala.foreach { partitionLocation =>
         val fileWriter = partitionLocation.asInstanceOf[WorkingPartition].getFileWriter
-        fileWriter.destroy(new IOException(s"Destroy FileWriter ${fileWriter} caused by " +
-          s"reserving slots failed for ${shuffleKey}."))
+        fileWriter.destroy(new IOException(s"Destroy FileWriter $fileWriter caused by " +
+          s"reserving slots failed for $shuffleKey."))
       }
       context.reply(ReserveSlotsResponse(StatusCode.RESERVE_SLOTS_FAILED, msg))
       return
@@ -213,7 +216,7 @@ private[deploy] class Controller(
             requestReplicaLocs.get(ind).getUniqueId)
         if (location == null) {
           location = requestReplicaLocs.get(ind)
-          val writer = storageManager.createWriter(
+          val writer = storageManager.createPartitionDataWriter(
             applicationId,
             shuffleId,
             location,
@@ -237,13 +240,13 @@ private[deploy] class Controller(
       logWarning(s"[handleReserveSlots] $msg, destroy writers.")
       primaryLocs.asScala.foreach { partitionLocation =>
         val fileWriter = partitionLocation.asInstanceOf[WorkingPartition].getFileWriter
-        fileWriter.destroy(new IOException(s"Destroy FileWriter ${fileWriter} caused by " +
-          s"reserving slots failed for ${shuffleKey}."))
+        fileWriter.destroy(new IOException(s"Destroy FileWriter $fileWriter caused by " +
+          s"reserving slots failed for $shuffleKey."))
       }
       replicaLocs.asScala.foreach { partitionLocation =>
         val fileWriter = partitionLocation.asInstanceOf[WorkingPartition].getFileWriter
-        fileWriter.destroy(new IOException(s"Destroy FileWriter ${fileWriter} caused by " +
-          s"reserving slots failed for ${shuffleKey}."))
+        fileWriter.destroy(new IOException(s"Destroy FileWriter $fileWriter caused by " +
+          s"reserving slots failed for $shuffleKey."))
       }
       context.reply(ReserveSlotsResponse(StatusCode.RESERVE_SLOTS_FAILED, msg))
       return
@@ -340,20 +343,19 @@ private[deploy] class Controller(
     future
   }
 
-  private def waitMapPartitionRegionFinished(fileWriter: FileWriter, waitTimeout: Long): Unit = {
-    if (fileWriter.isInstanceOf[MapPartitionFileWriter]) {
-      val delta = 100
-      var times = 0
-      while (delta * times < waitTimeout) {
-        if (fileWriter.asInstanceOf[MapPartitionFileWriter].isRegionFinished) {
+  private def waitMapPartitionRegionFinished(
+      fileWriter: PartitionDataWriter,
+      waitTimeout: Long): Unit = {
+    fileWriter match {
+      case writer: MapPartitionDataWriter =>
+        if (writer.checkPartitionRegionFinished(
+            waitTimeout)) {
           logDebug(s"CommitFile succeed to waitMapPartitionRegionFinished ${fileWriter.getFile.getAbsolutePath}")
-          return
+        } else {
+          logWarning(
+            s"CommitFile failed to waitMapPartitionRegionFinished ${fileWriter.getFile.getAbsolutePath}")
         }
-        Thread.sleep(delta)
-        times += 1
-      }
-      logWarning(
-        s"CommitFile faield to waitMapPartitionRegionFinished ${fileWriter.getFile.getAbsolutePath}")
+      case _ =>
     }
   }
 
@@ -424,11 +426,11 @@ private[deploy] class Controller(
 
     commitInfo.synchronized {
       if (commitInfo.status == CommitInfo.COMMIT_FINISHED) {
-        logInfo(s"${shuffleKey} CommitFinished, just return the response")
+        logInfo(s"$shuffleKey CommitFinished, just return the response")
         context.reply(commitInfo.response)
         return
       } else if (commitInfo.status == CommitInfo.COMMIT_INPROCESS) {
-        logInfo(s"${shuffleKey} CommitFiles inprogress, wait for finish")
+        logInfo(s"$shuffleKey CommitFiles inprogress, wait for finish")
         commitThreadPool.submit(new Runnable {
           override def run(): Unit = {
             waitForCommitFinish()
@@ -436,7 +438,7 @@ private[deploy] class Controller(
         })
         return
       } else {
-        logInfo(s"Start commitFiles for ${shuffleKey}")
+        logInfo(s"Start commitFiles for $shuffleKey")
         commitInfo.status = CommitInfo.COMMIT_INPROCESS
         workerSource.startTimer(WorkerSource.COMMIT_FILES_TIME, shuffleKey)
       }
@@ -529,10 +531,10 @@ private[deploy] class Controller(
           logInfo(
             s"CommitFiles for $shuffleKey success with " +
               s"${committedPrimaryIds.size()} committed primary partitions, " +
-              s"${emptyFilePrimaryIds.size()} empty primary partitions, " +
+              s"${emptyFilePrimaryIds.size()} empty primary partitions ${emptyFilePrimaryIds.asScala.mkString(",")}, " +
               s"${failedPrimaryIds.size()} failed primary partitions, " +
               s"${committedReplicaIds.size()} committed replica partitions, " +
-              s"${emptyFileReplicaIds.size()} empty replica partitions, " +
+              s"${emptyFileReplicaIds.size()} empty replica partitions ${emptyFileReplicaIds.asScala.mkString(",")} , " +
               s"${failedReplicaIds.size()} failed replica partitions.")
           CommitFilesResponse(
             StatusCode.SUCCESS,
