@@ -25,7 +25,6 @@ import java.util.concurrent._
 import java.util.concurrent.atomic.{AtomicBoolean, AtomicIntegerArray}
 
 import scala.collection.JavaConverters._
-import scala.util.Random
 
 import com.google.common.annotations.VisibleForTesting
 import io.netty.util.HashedWheelTimer
@@ -100,7 +99,9 @@ private[celeborn] class Worker(
         workerArgs.host,
         workerArgs.port,
         conf,
-        Math.min(64, Math.max(4, Runtime.getRuntime.availableProcessors())))
+        Math.min(64, Math.max(4, Runtime.getRuntime.availableProcessors())),
+        None,
+        Some(workerSource))
     } else {
       val externalSecurityContext = new RpcSecurityContextBuilder()
         .withServerSaslContext(
@@ -117,7 +118,8 @@ private[celeborn] class Worker(
         workerArgs.port,
         conf,
         Math.max(64, Runtime.getRuntime.availableProcessors()),
-        Some(externalSecurityContext))
+        Some(externalSecurityContext),
+        Some(workerSource))
     }
 
   private[worker] var internalRpcEnvInUse =
@@ -131,7 +133,9 @@ private[celeborn] class Worker(
         workerArgs.host,
         workerArgs.internalPort,
         conf,
-        Math.min(64, Math.max(4, Runtime.getRuntime.availableProcessors())))
+        Math.min(64, Math.max(4, Runtime.getRuntime.availableProcessors())),
+        None,
+        Some(workerSource))
     }
 
   private val host = rpcEnv.address.host
@@ -274,6 +278,7 @@ private[celeborn] class Worker(
   assert(replicatePort > 0, "worker replica bind port should be positive")
 
   storageManager.updateDiskInfos()
+  storageManager.startDeviceMonitor()
 
   // WorkerInfo's diskInfos is a reference to storageManager.diskInfos
   val diskInfos = JavaUtils.newConcurrentHashMap[String, DiskInfo]()
@@ -331,7 +336,7 @@ private[celeborn] class Worker(
       conf.workerCleanThreads)
   val asyncReplyPool: ScheduledExecutorService =
     ThreadUtils.newDaemonSingleThreadScheduledExecutor("worker-rpc-async-replier")
-  val timer = new HashedWheelTimer()
+  val timer = new HashedWheelTimer(ThreadUtils.namedSingleThreadFactory("worker-timer"))
 
   // Configs
   private val heartbeatInterval = conf.workerHeartbeatTimeout / 4
@@ -929,25 +934,27 @@ private[celeborn] class Worker(
   }
 
   ShutdownHookManager.get().addShutdownHook(
-    new Thread(new Runnable {
-      override def run(): Unit = {
-        logInfo("Shutdown hook called.")
-        workerStatusManager.exitEventType match {
-          case WorkerEventType.Graceful =>
-            shutdownGracefully()
-          case WorkerEventType.Decommission =>
-            decommissionWorker()
-          case _ =>
-            exitImmediately()
-        }
+    ThreadUtils.newThread(
+      new Runnable {
+        override def run(): Unit = {
+          logInfo("Shutdown hook called.")
+          workerStatusManager.exitEventType match {
+            case WorkerEventType.Graceful =>
+              shutdownGracefully()
+            case WorkerEventType.Decommission =>
+              decommissionWorker()
+            case _ =>
+              exitImmediately()
+          }
 
-        if (workerStatusManager.exitEventType == WorkerEventType.Graceful) {
-          stop(CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN)
-        } else {
-          stop(CelebornExitKind.EXIT_IMMEDIATELY)
+          if (workerStatusManager.exitEventType == WorkerEventType.Graceful) {
+            stop(CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN)
+          } else {
+            stop(CelebornExitKind.EXIT_IMMEDIATELY)
+          }
         }
-      }
-    }),
+      },
+      "worker-shutdown-hook-thread"),
     WORKER_SHUTDOWN_PRIORITY)
 
   @VisibleForTesting
