@@ -43,7 +43,7 @@ import org.apache.celeborn.common.metrics.source.{AbstractSource, ThreadPoolSour
 import org.apache.celeborn.common.network.util.{NettyUtils, TransportConf}
 import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType, StorageInfo}
 import org.apache.celeborn.common.quota.ResourceConsumption
-import org.apache.celeborn.common.util.{CelebornExitKind, CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
+import org.apache.celeborn.common.util.{CelebornExitKind, CelebornHadoopUtils, DiskUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 import org.apache.celeborn.service.deploy.worker._
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager.MemoryPressureListener
@@ -69,6 +69,9 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
 
   val storageExpireDirTimeout = conf.workerStorageExpireDirTimeout
 
+  val diskReserveSize = conf.workerDiskReserveSize
+  val diskReserveRatio = conf.workerDiskReserveRatio
+
   // (deviceName -> deviceInfo) and (mount point -> diskInfo)
   val (deviceInfos, diskInfos) = {
     val workingDirInfos =
@@ -145,8 +148,6 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     (flushers, totalThread)
   }
 
-  deviceMonitor.startCheck()
-
   val hdfsDir = conf.hdfsDir
   val hdfsPermission = new FsPermission("755")
   val (hdfsFlusher, _totalHdfsFlusherThread) =
@@ -278,7 +279,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         if (key.startsWith(SHUFFLE_KEY_PREFIX)) {
           val shuffleKey = parseDbShuffleKey(key)
           try {
-            val files = PbSerDeUtils.fromPbFileInfoMap(entry.getValue, cache)
+            val files = PbSerDeUtils.fromPbFileInfoMap(entry.getValue, cache, mountPoints)
             logDebug(s"Reload DB: $shuffleKey -> $files")
             diskFileInfos.put(shuffleKey, files)
             committedFileInfos.put(shuffleKey, files)
@@ -472,6 +473,24 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     null
   }
 
+  private def removeFileInfo(shuffleKey: String, fileName: String): FileInfo = {
+    val memoryShuffleMap = memoryFileInfos.get(shuffleKey)
+    if (memoryShuffleMap != null) {
+      return memoryShuffleMap.remove(fileName)
+    }
+
+    val diskShuffleMap = diskFileInfos.get(shuffleKey)
+    if (diskShuffleMap != null) {
+      if (workerGracefulShutdown) {
+        val committedFileInfoMap = committedFileInfos.get(shuffleKey)
+        committedFileInfoMap.remove(fileName)
+      }
+      return diskShuffleMap.remove(fileName)
+    }
+
+    null
+  }
+
   def getFetchTimeMetric(file: File): TimeWindow = {
     if (diskInfos != null) {
       val diskInfo = diskInfos.get(DeviceInfo.getMountPoint(file.getAbsolutePath, diskInfos))
@@ -493,14 +512,14 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         keyedWriters._1 -> keyedWriters._2.values().asScala.map(_.getFileLength).sum
       }
     }.toList.map { case (shuffleKey, usage) =>
-      shuffleKey.split("-")(0) -> usage
+      Utils.splitShuffleKey(shuffleKey)._1 -> usage
     }.groupBy(_._1).map { case (key, values) =>
       key -> values.map(_._2).sum
     }.toSeq.sortBy(_._2).reverse.take(conf.metricsAppTopDiskUsageCount * 2).toMap.asJava
   }
 
   def cleanFile(shuffleKey: String, fileName: String): Unit = {
-    cleanFileInternal(shuffleKey, getFileInfo(shuffleKey, fileName))
+    cleanFileInternal(shuffleKey, removeFileInfo(shuffleKey, fileName))
   }
 
   def cleanFileInternal(shuffleKey: String, fileInfo: FileInfo): Boolean = {
@@ -780,30 +799,55 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   }
 
   def updateDiskInfos(): Unit = this.synchronized {
-    disksSnapshot().filter(_.status != DiskStatus.IO_HANG).foreach { diskInfo =>
-      val totalUsage = diskInfo.dirs.map { dir =>
-        val writers = workingDirWriters.get(dir)
-        if (writers != null) {
-          writers.synchronized {
-            writers.values.asScala.map(_.getDiskFileInfo.getFileLength).sum
+    disksSnapshot()
+      .filter(diskInfo =>
+        diskInfo.status != DiskStatus.IO_HANG || diskInfo.status != DiskStatus.READ_OR_WRITE_FAILURE)
+      .foreach {
+        diskInfo =>
+          val totalUsage = diskInfo.dirs.map { dir =>
+            val writers = workingDirWriters.get(dir)
+            if (writers != null) {
+              writers.synchronized {
+                writers.values.asScala.map(_.getDiskFileInfo.getFileLength).sum
+              }
+            } else {
+              0
+            }
+          }.sum
+
+          try {
+            val (fileSystemReportedUsableSpace, fileSystemReportedTotalSpace) =
+              getFileSystemReportedSpace(diskInfo.mountPoint)
+            val actualReserveSize =
+              DiskUtils.getActualReserveSize(diskInfo, diskReserveSize, diskReserveRatio)
+            val workingDirUsableSpace =
+              Math.min(
+                diskInfo.configuredUsableSpace - totalUsage,
+                fileSystemReportedUsableSpace - actualReserveSize)
+            val usableSpace = Math.max(workingDirUsableSpace, 0)
+            logDebug(
+              s"Update diskInfo:${diskInfo.mountPoint} workingDirUsableSpace:$workingDirUsableSpace fileMeta:$fileSystemReportedUsableSpace " +
+                s"configuredUsableSpace:${diskInfo.configuredUsableSpace} totalUsage:$totalUsage totalSpace:$fileSystemReportedTotalSpace " +
+                s"actualReserveSize:$actualReserveSize usableSpace:$usableSpace")
+            diskInfo.setUsableSpace(usableSpace)
+            diskInfo.setTotalSpace(fileSystemReportedTotalSpace)
+          } catch {
+            case t: Throwable =>
+              logError(s"Update diskInfo:${diskInfo.mountPoint} failed.", t)
           }
-        } else {
-          0
-        }
-      }.sum
-      val fileSystemReportedUsableSpace = Files.getFileStore(
-        Paths.get(diskInfo.mountPoint)).getUsableSpace
-      val workingDirUsableSpace =
-        Math.min(diskInfo.configuredUsableSpace - totalUsage, fileSystemReportedUsableSpace)
-      logDebug(s"updateDiskInfos  workingDirUsableSpace:$workingDirUsableSpace filemeta:$fileSystemReportedUsableSpace conf:${diskInfo.configuredUsableSpace} totalUsage:$totalUsage")
-      diskInfo.setUsableSpace(workingDirUsableSpace)
-      diskInfo.setTotalSpace(totalUsage + workingDirUsableSpace)
-      diskInfo.updateFlushTime()
-      diskInfo.updateFetchTime()
-    }
+          diskInfo.updateFlushTime()
+          diskInfo.updateFetchTime()
+      }
     logInfo(s"Updated diskInfos:\n${disksSnapshot().mkString("\n")}")
   }
 
+  def getFileSystemReportedSpace(mountPoint: String): (Long, Long) = {
+    val fileStore = Files.getFileStore(Paths.get(mountPoint))
+    val fileSystemReportedUsableSpace = fileStore.getUsableSpace
+    val fileSystemReportedTotalSpace = fileStore.getTotalSpace
+    (fileSystemReportedUsableSpace, fileSystemReportedTotalSpace)
+  }
+
   def userResourceConsumptionSnapshot(): Map[UserIdentifier, ResourceConsumption] = {
     diskFileInfos.synchronized {
       // shuffleId -> (fileName -> fileInfo)
@@ -993,7 +1037,9 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
             case PartitionType.REDUCE =>
               new ReduceFileMeta(conf.shuffleChunkSize)
             case PartitionType.MAP =>
-              new MapFileMeta()
+              val mapFileMeta = new MapFileMeta()
+              mapFileMeta.setMountPoint(mountPoint)
+              mapFileMeta
             case PartitionType.MAPGROUP =>
               throw new NotImplementedError("Map group is not implemented")
           }
@@ -1033,6 +1079,11 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     }
     throw exception
   }
+
+  def startDeviceMonitor(): Unit = {
+    deviceMonitor.startCheck()
+  }
+
 }
 
 object StorageManager {
